@@ -1,13 +1,13 @@
 ;;; citar-denote.el --- Minor mode integrating Citar and Denote -*- lexical-binding: t -*-
 
-;; Copyright (C) 2022-2024 Peter Prevos
+;; Copyright (C) 2022-2025 Peter Prevos
 
 ;; Author: Peter Prevos <peter@prevos.net>
 ;; Maintainer: Peter Prevos <peter@prevos.net>
 ;; Homepage: https://github.com/pprevos/citar-denote
-;; Package-Version: 2.2.1
-;; Package-Revision: 4586e030445f
-;; Package-Requires: ((emacs "28.1") (citar "1.4") (denote "2.0") (dash "2.19.1"))
+;; Package-Version: 20251220.2230
+;; Package-Revision: 0c04d022ff2c
+;; Package-Requires: ((emacs "28.1") (citar "1.4") (denote "4.1") (dash "2.19.1"))
 
 ;; This file is NOT part of GNU Emacs.
 
@@ -51,7 +51,7 @@
 (defgroup citar-denote ()
   "Creating and accessing bibliography files with Citar and Denote."
   :group 'files
-  :link  '(url-link :tag "Homepage" "https://lucidmanager.org/productivity/bibliographic-notes-in-emacs-with-citar-denote/"))
+  :link  '(url-link :tag "Homepage" "https://github.com/pprevos/citar-denote/"))
 
 (defcustom citar-denote-keyword "bib"
   "Denote keyword (file tag) to indicate bibliographical notes.
@@ -85,8 +85,8 @@ text mode."
 (defcustom citar-denote-subdir nil
   "Save new bibliographic notes in a chosen or defined subdirectory.
 Options:
-- `nil', note is stored in `denote-directory'.
-- `t', Denote asks for subdirectory to store the note.
+- nil, note is stored in the default `denote-directory'.
+- t, Denote asks for subdirectory to store the note.
 - `string': When entering a string, the note is save in a subdirectory
   under `denote-directory'."
   ;; https://github.com/pprevos/citar-denote/issues/11
@@ -96,9 +96,11 @@ Options:
                   (string :tag "Save in named subdirectory")))
 
 (defcustom citar-denote-signature nil
-  "Ask for a signature, or use citation key when saving new bibliographic notes."
+  "Create a note with a signature (child sequence, citation key or free text).
+The option to create child notes requires the Denote-Sequence package."
   :group 'citar-denote
   :type  '(choice (const :tag "No signature" nil)
+                  (const :tag "Sequence" sequence)
                   (const :tag "Ask for signature" ask)
                   (const :tag "Use citation key as signature" citekey)))
 
@@ -116,7 +118,7 @@ Options:
 - `author-year': Author-year citation style, e.g. Stallman (1981)
 - `author-year-title': Combine author, year and title
 - `full': Full citation
-- `nil': BibTeX citekey
+- nil: BibTeX citekey
 
 For `author-year' and `author-year-title' you can configure:
 - `citar-denote-title-format-authors'
@@ -130,15 +132,29 @@ For `author-year' and `author-year-title' you can configure:
            (const :tag "Citekey" nil)))
 
 (defcustom citar-denote-title-format-authors 1
-  "Maximum number of authors in `author-year' for `citar-denote-title-format'."
+  "Maximum number of authors befor et al. in `citar-denote--format-author-editor'."
   :group 'citar-denote
   :type  'integer)
 
 (defcustom citar-denote-title-format-andstr "and"
-  "Connecting word for authors in `author-year' for `citar-denote-note-title'."
+  "Connecting word for last two authors `citar-denote--format-author-editor'."
   :group 'citar-denote
   :type  'string)
 
+(defcustom citar-denote-cite-includes-reference nil
+  "Include reference notes in cite search.
+When non-nil, searching for files citing a bibtex key includes Denote files
+that only contain the citekey in the reference front matter (and not as a
+@-style citation)."
+  ;; https://github.com/pprevos/citar-denote/issues/34
+  :group 'citar-denote
+  :type  'boolean)
+
+(defcustom citar-denote-open-attachment t
+  "When non-nil, open the first defined attachment of the bibliographic entry."
+  :group 'citar-denote
+  :type 'boolean)
+
 (defvar citar-denote-file-types
   `((org
      :reference-format "#+reference:  %s\n"
@@ -201,18 +217,33 @@ citation key CITEKEY."
 			                           (denote-keywords)))))))
     (append keywords (list citar-denote-keyword))))
 
-(defun citar-denote--add-reference (citekey file-type)
-  "Add reference with CITEKEY in front matter of the file with FILE-TYPE.
+(defun citar-denote--add-reference (citekeys)
+  "Add CITEKEYS to the front matter of the current buffer.
+`citar-denote-add-reference' is the interactive version of this function."
+  (let* ((file (buffer-file-name))
+         (filetype (denote-filetype-heuristics file))
+         (new-references (mapconcat #'identity citekeys ";"))
+         (existing-references (citar-denote--retrieve-references file)))
+    (if existing-references
+        ;; Add to existing
+        (save-excursion
+          (goto-char (point-min))
+          (re-search-forward (citar-denote--reference-regex filetype))
+          (end-of-line)
+          (insert (concat ";" new-references))
+          (save-buffer))
+      ;; New reference line
+      (citar-denote--add-new-reference-line citekeys filetype))))
 
-`citar-denote-add-citekey' is the interactive version of this function."
+(defun citar-denote--add-new-reference-line (citekeys filetype)
+  "Add a new reference line with CITEKEYS to buffer of FILETYPE."
   (save-excursion
     (goto-char (point-min))
-    (re-search-forward "^\n" nil t)
-    (forward-line -1)
-    (if (not (eq (or file-type 'org) 'org))
-        (forward-line -1))
-    (insert
-     (format (citar-denote--reference-format file-type) citekey))))
+    (while (not (looking-at "^$")) ;; Search for the first empty line
+      (forward-line 1))
+    (when (not (eq filetype 'org)) (forward-line -1))
+    (insert (format (citar-denote--reference-format filetype)
+                    (mapconcat #'identity citekeys ";")))))
 
 (defun citar-denote--retrieve-references (file)
   "Return reference citekey(s) from FILE front matter."
@@ -233,29 +264,39 @@ citation key CITEKEY."
 
 If CITEKEYS is omitted, return all Denote files tagged with
 `citar-denote-keyword'."
-  (let ((files (make-hash-table :test 'equal)))
-    (prog1 files
-      (dolist (file (denote-directory-files
-                     (concat "_" citar-denote-keyword)))
+  (let ((files (denote-directory-files (concat "_" citar-denote-keyword)))
+	(notes (make-hash-table :test 'equal)))
+    (prog1 notes
+      (dolist (file files)
         (let ((keys-in-file (citar-denote--retrieve-references file)))
           (dolist (key keys-in-file)
             (if citekeys
                 (dolist (k citekeys)
                   (when (string= k key)
-                    (push file (gethash key files))))
-              (push file (gethash key files))))))
+                    (push file (gethash key notes))))
+              (push file (gethash key notes))))))
       (maphash
        (lambda (key filelist)
-         (puthash key (nreverse filelist) files)) files))))
+         (puthash key (nreverse filelist) notes))
+       notes))))
 
 (defun citar-denote--retrieve-cite-files (citekey)
-  "Return names of Denote files that contain CITEKEY."
-  (let ((files (denote-directory-files nil nil t)))
+  "Return names of Denote files that contain CITEKEY.
+
+If `citar-denote-cite-includes-reference' is non-nil, the results
+will include Denote files with CITEKEY only in the reference
+front matter."
+  (let ((cite-sign (if citar-denote-cite-includes-reference "" "@"))
+        ;; Include '@' in the xref query only when front
+        ;; matter '#+reference:' to bibtex keyword should
+        ;; not be included in the results
+        (files (denote-directory-files nil nil t)))
     (delete-dups
      (mapcar
       #'xref-location-group
       (mapcar #'xref-match-item-location
-              (xref-matches-in-files (format "@%s" citekey) files))))))
+              (xref-matches-in-files
+               (format "%s%s" cite-sign citekey) files))))))
 
 (defun citar-denote--has-notes ()
   "Return a list of all citekeys referenced in a Denote file.
@@ -267,7 +308,6 @@ See documentation for `citar-has-notes'."
 
 (defun citar-denote--has-citekeys (citekeys)
   "Return hash table of citar entries with associated CITEKEYS."
-  ;; TODO: Is this function doing the same thing as cetar-denote--get-notes?
   (let ((citar-entries (citar-get-entries))
         (citekey-entries (make-hash-table :test 'equal)))
     (mapc (lambda (key)
@@ -278,13 +318,23 @@ See documentation for `citar-has-notes'."
 
 (defun citar-denote--remove-bibkey (file)
   "Remove `citar-denote-bibkey' file tag from FILE."
+  ;; https://github.com/pprevos/citar-denote/issues/43
   (let* ((file-type (denote-filetype-heuristics file))
-         (keywords (denote-retrieve-keywords-value file file-type)))
-    (denote-rewrite-keywords
-     file
-     (delete citar-denote-keyword keywords)
-     file-type)
-    (denote-rename-file-using-front-matter file t)))
+         (keywords (denote-retrieve-keywords-value file file-type))
+         (new-keywords (delete citar-denote-keyword keywords)))
+    (denote-rewrite-keywords file new-keywords file-type)
+    (denote-rename-file-using-front-matter file)))
+
+(defun citar-denote--add-bibkey (file)
+  "Add `citar-denote-bibkey' file tag from FILE."
+  ;; https://github.com/pprevos/citar-denote/issues/44
+  (when-let* ((file-type (denote-filetype-heuristics file))
+              (keywords (string-split (denote-retrieve-filename-keywords file) "_"))
+              (existing-p (not (member citar-denote-keyword keywords)))
+              (new-keywords (delete-dups (denote-keywords-sort
+                                          (cons citar-denote-keyword keywords)))))
+    (denote-rewrite-keywords file new-keywords file-type)
+    (denote-rename-file-using-front-matter file)))
 
 (defun citar-denote--extract-citations-blocks (line)
   "Extract all Org mode citation blocks from LINE."
@@ -309,16 +359,28 @@ See documentation for `citar-has-notes'."
   "Extract citations from all Denote files."
   ;; Extract lines with citations
   (let* ((xrefs (xref-matches-in-files "\\[cite:@.*\\]"
-                 (denote-directory-files nil nil t)))
+                                       (denote-directory-files nil nil t)))
          (citation-lines (mapcar #'substring-no-properties
                                  (mapcar #'xref-match-item-summary
                                          xrefs)))
          (citation-blocks (mapcar #'citar-denote--extract-citations-blocks
-				       citation-lines))
+				  citation-lines))
 	 (citation-blocks-flat  (apply #'append citation-blocks))
 	 (citations (mapcar #'citar-denote--extract-citation-keys
 			    citation-blocks-flat)))
-	 (delete-dups (apply #'append citations))))
+    (delete-dups (apply #'append citations))))
+
+(defun citar-denote--format-author-editor (citekey)
+  "Extract author or editor from CITEKEY.
+Format author(s) or editor(s) in accordance with:
+- `citar-denote-title-format-authors': Number of authors before et al.
+- `citar-denote-title-format-andstr': Connecting string between last two authors."
+  (let ((author-names (or (citar-get-value "author" citekey)
+                          (citar-get-value "editor" citekey))))
+    (citar--shorten-names
+     author-names
+     citar-denote-title-format-authors
+     citar-denote-title-format-andstr)))
 
 (defun citar-denote--generate-title (citekey)
   "Generate title for new bibliographic note using CITEKEY.
@@ -326,20 +388,15 @@ See documentation for `citar-has-notes'."
 Either title, author/year, full reference or citation key.
 Based on the `citar-denote-title-format' variable."
   ;; https://github.com/pprevos/citar-denote/issues/15
-  (let ((title (citar-get-value "title" citekey))
-        (author-names (or (citar-get-value "author" citekey)
-                          (citar-get-value "editor" citekey)))
+  (let ((author-editor (citar-denote--format-author-editor citekey))
+        (title (citar-get-value "title" citekey))
         (year (or (citar-get-value "year" citekey)
                   (citar-get-value "date" citekey)
                   (citar-get-value "issued" citekey))))
     (cond ((equal citar-denote-title-format "title")
            title)
           ((equal citar-denote-title-format "author-year")
-           (concat (citar--shorten-names
-                    author-names
-                    citar-denote-title-format-authors
-                    citar-denote-title-format-andstr)
-                   " (" year ")"))
+           (concat author-editor " (" year ")"))
           ((equal citar-denote-title-format "author-year-title")
            (let* ((citar-denote-title-format "author-year")
                   (author-year (citar-denote--generate-title citekey))
@@ -350,34 +407,73 @@ Based on the `citar-denote-title-format' variable."
              (substring ref 0 (- (length ref) 2))))
           (t citekey))))
 
+(defun citar-denote--expand-citation-macros (citekey)
+  "Expand Citar-Denote citation macros for CITEKEY.
+
+Citation macros expand to information from the bibliographic entry and
+are in the form of %^{field-name}, for example %^{title}. Any field can be
+used as a macro.
+
+There are four special citation macros:
+- %^{author-or-editor}: Author(s) or Editor(s) formatted with
+  `citar-denote--format-author-editor'.
+- %^{doi-url}: DOI or URL. When both are defined, use DOI.
+- %^{file}: First listed file (links only work with absolute file path).
+- %^{citation}: Full citation."
+  ;; Define special citation macros
+  (let* ((author-or-editor (citar-denote--format-author-editor citekey))
+         (url-value (citar-get-value "url" citekey))
+         (doi-value (citar-get-value "doi" citekey))
+         (doi-url (cond
+                   (doi-value (concat "doi:" doi-value))
+                   (url-value url-value)
+                   (t nil)))
+         (files (citar-get-value "file" citekey))
+         (file (when files (car (split-string files ";"))))
+         (citation (citar-format-reference (list citekey)))
+         ;; All defined bibtex fields
+         (entry (citar-get-entry citekey))
+         ;; Create a mapping of macro names to values
+         (macro-map `(("author-or-editor" . ,author-or-editor)
+                      ("doi-url" . ,doi-url)
+                      ("file" . ,file)
+                      ("citation" . ,citation))))
+    (save-excursion
+      (goto-char (point-min))
+      ;; Macro format: %^{field-name}
+      (while (re-search-forward "\\%^{\\([^}]+\\)}" nil t)
+        (let* ((macro-name (match-string 1))
+               (replacement (or (cdr (assoc-string macro-name macro-map 'case-fold))
+                                (cdr (assoc-string macro-name entry 'case-fold)))))
+          (when replacement
+            (replace-match replacement t t)))))))
+
 (defun citar-denote--get-nocite ()
   "Select item(s) from Citar entries not cited or referenced in Denote files."
-  (if-let* ((bibliography (hash-table-keys (citar-get-entries)))
-            (citations (citar-denote--extract-citations))
-            (references (hash-table-keys (citar-denote--get-notes)))
-	    (union (cl-union citations references))
-	    (nocite (cl-set-difference bibliography union :test #'string=)))
-      (citar-select-refs
-       :multiple t
-       :filter (citar-denote--has-citekeys nocite))
-    nil))
+  (let* ((bibliography (hash-table-keys (citar-get-entries)))
+         (citations (citar-denote--extract-citations))
+         (references (hash-table-keys (citar-denote--get-notes)))
+	 (cite-ref (cl-union citations references))
+	 (nocite (cl-set-difference bibliography cite-ref :test #'string=)))
+    (when nocite (citar-select-refs
+                  :multiple t
+                  :filter (citar-denote--has-citekeys nocite)))))
 
 (defun citar-denote--get-non-referenced (file)
   "Select Citar entry not already referenced in FILE."
-  (let* ((all-items (hash-table-keys (citar-get-entries)))
+  (let* ((bibliography (hash-table-keys (citar-get-entries)))
          (references (citar-denote--retrieve-references file))
-         (unused (-difference all-items references)))
-    (citar-select-refs
-     :multiple t
-     :filter (citar-denote--has-citekeys unused))))
+         (unused (-difference bibliography references)))
+    (when unused (citar-select-refs
+                  :multiple t
+                  :filter (citar-denote--has-citekeys unused)))))
 
 (defun citar-denote--select-file-using-title (files)
-  "Select a file name based on a list of note titles from a list of FILES."
+  "Select a file based on a list of note titles from a list of FILES."
   (let* ((description-file-alist
           (mapcar
            (lambda (file)
-             (let ((file-type (denote-filetype-heuristics file)))
-               (cons (denote--link-get-description file) file)))
+             (cons (denote--link-get-description file) file))
            files))
          (selected-description (completing-read "Select note: " description-file-alist))
          (selected-file (cdr (assoc selected-description description-file-alist))))
@@ -386,35 +482,46 @@ Based on the `citar-denote-title-format' variable."
 (defun citar-denote--create-note (citekey &optional _entry)
   "Create a bibliographic note for CITEKEY with properties ENTRY.
 
-The note file type is determined by `citar-denote-file-type'.
-
-The title format is set by `citar-denote-title-format'.
-
-When `citar-denote-subdir' is non-nil, prompt for a subdirectory.
-
-When `citar-denote-signature' is non-nil, prompt for a signature or
-use citation key."
+- The note file type is determined by `citar-denote-file-type'.
+- The title format is set by `citar-denote-title-format'.
+- When `citar-denote-subdir' is non-nil, prompt for a subdirectory.
+- When `citar-denote-template' is a symbol, use the specified template,
+  if otherwise non-nil, prompt for a Denote template. Citation macros are
+  expanded with `citar-denote--expand-citation-macros'.
+- When `citar-denote-signature' is non-nil, select a parent, prompt for a
+  signature or use the citation key."
   (denote
    (read-string "Title: " (citar-denote--generate-title citekey))
    (citar-denote--keywords-prompt citekey)
    citar-denote-file-type
    (when citar-denote-subdir
      (if (stringp citar-denote-subdir)
-         (expand-file-name
-          (concat denote-directory citar-denote-subdir))
+         (expand-file-name citar-denote-subdir denote-directory)
        (denote-subdirectory-prompt)))
    nil
    (when citar-denote-template
-     (if (not (eq citar-denote-template))
-         citar-denote-template
-       (denote-template-prompt)))
-   (cond ((eq citar-denote-signature 'ask)
+     (or (alist-get citar-denote-template denote-templates)
+         (denote-template-prompt)))
+   (cond ((eq citar-denote-signature 'sequence)
+          (if (locate-library "denote-sequence")
+              (let ((parent (denote-retrieve-filename-signature
+                            (denote-sequence-file-prompt
+                             "New child of SEQUENCE"))))
+                (denote-sequence-get-new 'child parent))
+            (user-error "Please modify `citar-denote-signature' of install Denote-Sequence")))
+         ((eq citar-denote-signature 'ask)
           (denote-signature-prompt nil "Signature: "))
          ((eq citar-denote-signature 'citekey)
           citekey)
-         (nil nil))
-   )
-  (citar-denote--add-reference citekey citar-denote-file-type))
+         (nil nil)))
+  (citar-denote--add-new-reference-line (list citekey) citar-denote-file-type)
+  (when citar-denote-template (citar-denote--expand-citation-macros citekey))
+  ;; Open available atachment in other window
+  (when (and citar-denote-open-attachment (citar-get-value "file" citekey))
+    (when (one-window-p)
+      (split-window-right))
+    (other-window 1)
+    (citar-open-files citekey)))
 
 ;; Interactive functions
 
@@ -430,23 +537,26 @@ Provides a selection list of all bibliographic entries with notes."
     (find-file (cdr file))))
 
 ;;;###autoload
-(defun citar-denote-find-citation ()
-  "Find a Denote file that cites a bibliographic entry.
+(defun citar-denote-find-citation (citekey)
+  "Find Denote files that cite a bibliographic entry CITEKEY.
 
-Provides a selection list of all bibliographic entries cited in Denote files."
-  (interactive)
-  (if-let* ((citations (citar-denote--extract-citations))
-            (citekey (citar-select-ref
-                      :filter (citar-denote--has-citekeys citations)))
-            (files (citar-denote--retrieve-cite-files citekey)))
+When called interactively, select an entry from a list with all
+bibliographic entries cited in Denote files."
+  (interactive
+   (if-let* ((citations (citar-denote--extract-citations))
+             (citekey (citar-select-ref
+                       :filter (citar-denote--has-citekeys citations))))
+       (list citekey)
+     (user-error "No citations found in Denote files")))
+  (if-let ((files (citar-denote--retrieve-cite-files citekey)))
       (progn (find-file (denote-get-path-by-id
                          (denote-extract-id-from-string
                           (if (= (length files) 1)
                               (car files)
-                            (denote-link--find-file-prompt files)))))
+                            (denote-select-linked-file-prompt files)))))
              (goto-char (point-min))
              (search-forward citekey))
-    (message "Not citations found in Denote file")))
+    (message "No citations of %s found in Denote files" citekey)))
 
 ;;;###autoload
 (defun citar-denote-dwim ()
@@ -455,15 +565,16 @@ Provides a selection list of all bibliographic entries cited in Denote files."
 When more than one bibliographic item is referenced, select item first."
   (interactive)
   ;; Any citation keys in the note?
-  (if-let* ((keys (citar-denote--retrieve-references (buffer-file-name)))
+  (if-let* ((buffer-name (buffer-file-name))
+            (keys (citar-denote--retrieve-references buffer-name))
             (key (if (= (length keys) 1)
                      (car keys)
                    (citar-select-ref
                     :filter (citar-denote--has-citekeys keys)))))
       (citar-open (list key))
-    (if (denote-file-is-note-p (buffer-file-name))
-        (when (yes-or-no-p "Current buffer does not reference a citation key.
-Add a reference? ")
+    (if (and file (denote-file-is-note-p (buffer-file-name)))
+        (when (yes-or-no-p
+               "Current note does contain a reference. Add one? ")
           (citar-denote-add-citekey)
           (citar-denote-dwim))
       (message "Buffer is not a Denote file"))))
@@ -474,74 +585,60 @@ Add a reference? ")
 
 When more than one bibliographic item is referenced, select item first."
   (interactive)
-  (if-let* ((keys (citar-denote--retrieve-references (buffer-file-name)))
+  (if-let* ((buffer (buffer-file-name))
+            (keys (citar-denote--retrieve-references buffer))
             (key (if (= (length keys) 1)
                      (car keys)
                    (citar-select-ref
                     :filter (citar-denote--has-citekeys keys)))))
       (citar-open-entry key)
-    (if (denote-file-is-note-p (buffer-file-name))
-        (when (yes-or-no-p "Current buffer does not reference a citation key.
-Add a reference? ")
-          (citar-denote-add-citekey)
-          (citar-denote-open-reference-entry))
-      (message "Buffer is not a Denote file"))))
+    (message "Buffer is not a Denote file or has no reference(s)")))
 
 ;;;###autoload
-(defun citar-denote-add-citekey ()
+(defun citar-denote-add-reference (&optional nocite)
   "Add citation key(s) to existing note.
-
-Convert note to a bibliographic note when no existing reference exists."
-  (interactive)
+With universal argument choose from entries not yet used in Denote (NOCITE).
+Add `citar-denote-bibkey' keyword when no existing reference exists."
+  (interactive "P")
   (if-let* ((file (buffer-file-name))
-            (file-type (denote-filetype-heuristics file))
-            ((denote-file-is-note-p file))
-            (citekeys (citar-denote--get-non-referenced file))
-            (references (mapconcat 'identity citekeys ";")))
-      ;; Check whether reference line already exists
-      (if-let (keys (citar-denote--retrieve-references file))
-          ;; Append existing reference
-          (save-excursion
-            (goto-char (point-min))
-            (re-search-forward (citar-denote--reference-regex file-type))
-            (end-of-line)
-            (insert (concat ";" references))
-            (save-buffer))
-        ;; Add new reference line
-        (progn (citar-denote--add-reference references file-type)
-               (denote-keywords-add (list citar-denote-keyword))
-               (save-buffer)))
-    (message "Buffer is not a Denote file")))
+            (denote-p (denote-file-is-note-p file))
+            (citekeys (if nocite
+                          (citar-denote--get-nocite)
+                        (citar-denote--get-non-referenced file))))
+      (progn
+        (citar-denote--add-reference citekeys)
+        (citar-denote--add-bibkey file)
+        (save-buffer))
+    (if (not denote-p)
+        (message "Buffer is not a Denote file")
+      (message "All bibliogary entries have been cited or referenced"))))
 
 ;;;###autoload
-(defun citar-denote-remove-citekey ()
+(defun citar-denote-remove-reference ()
   "Remove a reference from a bibliographic note.
-
-If the only or last reference is removed, remove `_bib' keyword."
+If the only or last reference is removed, also remove `citar-denote-keyword'."
   (interactive)
   (if-let* ((file (buffer-file-name))
-            (file-type (denote-filetype-heuristics file))
-	    (citekeys (citar-denote--retrieve-references file))
-	    (selected (if (< (length citekeys) 2)
-			  (car citekeys)
-			(citar-select-ref
-			 :filter
-			 (citar-denote--has-citekeys citekeys)))))
+            (filetype (denote-filetype-heuristics file))
+            (citekeys (citar-denote--retrieve-references file))
+            (selected (if (< (length citekeys) 2)
+                          (car citekeys)
+                        (citar-select-ref
+                         :filter
+                         (citar-denote--has-citekeys citekeys)))))
       (let ((new-citekeys (delete selected citekeys)))
         (save-excursion
-	  ;; Remove references line
-	  (goto-char (point-min))
-	  (re-search-forward
-	   (citar-denote--reference-regex file-type))
-	  (move-beginning-of-line nil)
-	  (kill-line 1)
-	  ;; Add new line or remove file tags when applicable
-	  (if (> (length new-citekeys) 0)
-	      (citar-denote--add-reference
-	       (mapconcat 'identity new-citekeys ";")
-               file-type)
-	    (citar-denote--remove-bibkey file))
-	  (save-buffer)))
+          ;; Remove references line
+          (goto-char (point-min))
+          (re-search-forward
+           (citar-denote--reference-regex filetype))
+          (move-beginning-of-line nil)
+          (kill-line 1)
+          ;; Add new line or remove file tags when applicable
+          (if (> (length new-citekeys) 0)
+              (citar-denote--add-new-reference-line new-citekeys filetype)
+            (citar-denote--remove-bibkey file))
+          (save-buffer)))
     (message "No references in this buffer, or not a Denote file")))
 
 ;;;###autoload
@@ -551,7 +648,7 @@ If the only or last reference is removed, remove `_bib' keyword."
 When more than one bibliographic item is referenced, select item first."
   (interactive)
   (let ((file (buffer-file-name)))
-    (if (denote-file-is-note-p file)
+    (if (and file (denote-file-is-note-p file))
         (let* ((citekeys (citar-denote--retrieve-references file))
                (citekey (when citekeys
                           (if (= (length citekeys) 1)
@@ -564,12 +661,12 @@ When more than one bibliographic item is referenced, select item first."
            (files
             (find-file (denote-get-path-by-id
                         (denote-extract-id-from-string
-                         (denote-link--find-file-prompt files))))
+                         (denote-select-from-files-prompt files))))
             (goto-char (point-min))
             (search-forward citekey))
            ((null citekey)
             (message "This is not a bibliographic note"))
-           (t (message "No citation found in other Denote files"))))
+           (t (message "Reference not cited in Denote files"))))
       (message "Buffer is not a Denote file"))))
 
 ;;;###autoload
@@ -595,9 +692,11 @@ When more than one bibliographic item is referenced, select item first."
 
 ;;;###autoload
 (defun citar-denote-nocite ()
-  "Open Citar with bibliographic entries not cited or referenced in Denote."
+  "Open bibliographic entries not cited or referenced in Denote files with Citar."
   (interactive)
-  (citar-open (citar-denote--get-nocite)))
+  (if-let ((citekeys (citar-denote--get-nocite)))
+      (citar-open citekeys)
+    (message "All bibliogary entries have been cited or referenced")))
 
 ;;;###autoload
 (defun citar-denote-cite-nocite ()
@@ -622,15 +721,56 @@ When more than one bibliographic item is referenced, select item first."
              (length nobib)
              (mapconcat #'identity nobib ", "))))
 
+;;;###autoload
+(defun citar-denote-check-keywords ()
+  "Check that each note with references has a bib keyword.
+Remove bib keyword when no reference, but `citar-denote-keyword' is present.
+Add bib keyword when reference is present, but `citar-denote-keyword' is missing."
+  (interactive)
+  (let ((files (denote-directory-files nil nil t)))
+    (dolist (file files)
+      (let* ((file-type (denote-filetype-heuristics file))
+             (regex (citar-denote--reference-regex file-type))
+             (reference-p (with-temp-buffer
+                            (insert-file-contents file)
+                            (re-search-forward regex nil t)))
+             (keywords (denote-retrieve-front-matter-keywords-value file file-type))
+             (bib-keyword-p (not (null (member citar-denote-keyword keywords)))))
+        ;; Remove bib keyword when no reference but citar-denote-keyword
+        (when (and (not reference-p) bib-keyword-p)
+          (citar-denote--remove-bibkey file))
+        ;; Add bib keyword when reference but no citar-denote-keyword
+        (when (and reference-p (not bib-keyword-p))
+          (citar-denote--add-bibkey file))))))
+
+;;;###autoload
+(defun citar-denote-create-silo-note (silo)
+  "Select a SILO and run `citar-create-denote'.
+SILO is a file path from `denote-silo-directories'.
+
+When called from Lisp, SILO is a directory path."
+  ;; https://github.com/pprevos/citar-denote/issues/41
+  (interactive (if (locate-library "denote-silo")
+		   (list (denote-silo-directory-prompt))
+		 (user-error "Denote-Silo package not installed")))
+  (let ((denote-directory silo))
+    (call-interactively #'citar-create-note)))
+
 (define-obsolete-function-alias
   'citar-denote-find-nocite
-  'citar-denote-cite-nocite
-  "1.6")
+  'citar-denote-cite-nocite "1.6")
 
 (define-obsolete-function-alias
   'citar-denote-reference-nocite
-  'citar-denote-nocite
-  "1.8.1")
+  'citar-denote-nocite "1.8.1")
+
+(define-obsolete-function-alias
+  'citar-denote-add-citekey
+  'citar-denote-add-reference "2.3")
+
+(define-obsolete-function-alias
+  'citar-denote-remove-citekey
+  'citar-denote-remove-reference "2.3")
 
 ;; Citar integration
 
